/* read time stamp from .his file  */
/* usage is 'his_time inputfile' */

#include<stdio.h>
#include<stdlib.h>
#include<string.h>

#define INTEL
#define HIS_Header_Size 64

struct HIS_Header
{
	char			head[2];			/*0-1*/
	short			comment_length;		/*2-3*/
	short			width;				/*4-5*/
	short			height;				/*6-7*/
	short			x_offset;			/*8-9*/
	short			y_offset;			/*10-11*/
	short			type;				/*12-13*/
	unsigned short			n_image1;			/*14-17*/
	unsigned short			n_image2;			/*14-17*/
	short			reserve1;			/*18-19*/
	short			reserve2;			/*20-21*/
//	double			time_stamp;			/*22-29*/
	unsigned char			time_s1;			/*22*/
	unsigned char			time_s2;			/*23*/
	unsigned char			time_s3;			/*24*/
	unsigned char			time_s4;			/*25*/
	unsigned char			time_s5;			/*26*/
	unsigned char			time_s6;			/*27*/
	unsigned char			time_s7;			/*28*/
	unsigned char			time_s8;			/*29*/
	unsigned short			maker1;				/*30-31*/
	unsigned short			maker2;				/*32-33*/
	char			reserved[30];		/*34-63*/
	char			*comment;
};
typedef struct HIS_Header HISHeader;


unsigned short	*data;
unsigned char	*cdata;
long			NP;

FILE	*fi;

/*----------------------------------------------------------------------*/
int read_hishead(fname,his)
HISHeader	*his;
char *fname;
{
	int		j;
	FILE	*f;

//open input files
	if((f = fopen(fname,"rb")) == NULL){
		printf("can not open %s for input\n", fname);
		free(his->comment); 
		return(-20);
	}
// read comment and image data from file1
	if (fread(his, sizeof(char), HIS_Header_Size, f) != HIS_Header_Size){
		fprintf(stderr, "EOF in input file (header)\n"); free(his->comment); return(-1);
	}
	if (strncmp(his->head, "IM", 2)){
		fclose(fi); free(his->comment);
		fprintf(stderr, "This is not HIS image file.\n"); 
		return(-21);
	}

	his->comment=(char *)malloc(his->comment_length+1);
	if (his->comment == NULL){
		fclose(f); free(his->comment); 
		fprintf(stderr, "Connot allocate memory (HIS_Header.comment)\n"); 
		return(-22);
	}
	if (fread(his->comment, sizeof(char), his->comment_length, f) != his->comment_length){
		fclose(f); free(his->comment); 
		fprintf(stderr, "EOF found (comment)\n"); 
		return(-23);
	}
	his->comment[his->comment_length] = '\0';

// check for correct reading of HIS file
//printf("-------------------------------\n");
//printf("%s\n", his->head);				/*0-1*/
//printf("%d\t", his->comment_length);		/*2-3*/
//printf("%d\t", his->width);				/*4-5*/
//printf("%d\t", his->height);				/*6-7*/
//printf("%d\t", his->x_offset);			/*8-9*/
//printf("%d\t", his->y_offset);			/*10-11*/
//printf("%d\t", his->type);				/*12-13*/
//printf("%d\t", his->n_image1);			/*14-15*/
//printf("%ld\t", his->n_image2*65536);		/*16-17*/
//printf("%d\t", his->reserve1);			/*18-19*/
//printf("%d\t", his->reserve2);			/*20-21*/
//printf("%lf\t", his->time_stamp);			/*22-29*/
//printf("%d\t", his->maker1);				/*30-31*/
//printf("%d\t", his->maker2);				/*32-33*/
//printf("%s\n", his->reserved);			/*34-64*/
//printf("%s\n", his->comment);
//printf("-------------------------------\n");
	
	
	fclose(f);
	return(0);
}

/*----------------------------------------------------------------------*/
int read_his(his)
HISHeader	*his;
{
	int		j;

// read comment and image data from file1
	if (fread(his, sizeof(char), HIS_Header_Size, fi) != HIS_Header_Size){
		fprintf(stderr, "EOF in input file (header)\n"); free(his->comment); return(-1);
	}
	if (strncmp(his->head, "IM", 2)){
		fclose(fi); free(his->comment);
		fprintf(stderr, "This is not HIS image file.\n"); 
		return(-11);
	}

	his->comment=(char *)malloc(his->comment_length+1);
	if (his->comment == NULL){
		fclose(fi); free(his->comment); 
		fprintf(stderr, "Connot allocate memory (HIS_Header.comment)\n"); 
		return(-12);
	}
	if (fread(his->comment, sizeof(char), his->comment_length, fi) != his->comment_length){ 
		fclose(fi); free(his->comment); 
		fprintf(stderr, "EOF found (comment)\n"); 
		return(-13);
	}
//	his->comment[his->comment_length] = '\0';

	
	if(his->type==2){
		if ((j = fread(data, sizeof(unsigned short), NP, fi)) != NP){
			fclose(fi); free(his->comment); 
			fprintf(stderr, " Error reading at %d\n",  j);
			return(-14);
		}
	}
	if(his->type==6){
		if ((j = fread(cdata, sizeof(unsigned char), NP, fi)) != NP){
			fclose(fi); free(his->comment); 
			fprintf(stderr, " Error reading at %d\n",  j);
			return(-14);
		}
	}
	
		return(0);
	}

/*----------------------------------------------------------------------*/

double get_times(his)
HISHeader	*his;
{
	double dbltime;
	FILE	*fo;
	char	outfile[20];
	int		i;
	
	sprintf(outfile, ("temp.dat"));
	//open temp files
	if((fo = fopen(outfile,"wb")) == NULL){
		printf("can not open temp.dat\n");
		return(-10);
	}
	if (i = fwrite(&his->time_s1, sizeof(char), 1, fo) > sizeof(char)){
		printf("can not write1 temp.dat\n");
		return(-11);
	}
	if (i = fwrite(&his->time_s2, sizeof(char), 1, fo) > sizeof(char)){
		printf("can not write2 temp.dat\n");
		return(-11);
	}
	if (i = fwrite(&his->time_s3, sizeof(char), 1, fo) > sizeof(char)){
		printf("can not write3 temp.dat\n");
		return(-11);
	}
	if (i = fwrite(&his->time_s4, sizeof(char), 1, fo) > sizeof(char)){
		printf("can not write4 temp.dat\n");
		return(-11);
	}
	if (i = fwrite(&his->time_s5, sizeof(char), 1, fo) > sizeof(char)){
		printf("can not write5 temp.dat\n");
		return(-11);
	}
	if (i = fwrite(&his->time_s6, sizeof(char), 1, fo) > sizeof(char)){
		printf("can not write6 temp.dat\n");
		return(-11);
	}
	if (i = fwrite(&his->time_s7, sizeof(char), 1, fo) > sizeof(char)){
		printf("can not write7 temp.dat\n");
		return(-11);
	}
	if (i = fwrite(&his->time_s8, sizeof(char), 1, fo) > sizeof(char)){
		printf("can not write8 temp.dat\n");
		return(-11);
	}
	fclose(fo);

	if((fo = fopen(outfile,"rb")) == NULL){
		printf("can not open temp.dat\n");
		return(-10);
	}
	if (fread(&dbltime, sizeof(double), 1, fo) != 1){
		printf("cannot read\n");
		return(-1);
	}
	fclose(fo);
	if( remove(outfile) == 0 ){
	}
	else{
		printf( "failed to delete %s.", outfile);
	}

	return dbltime;
}

int	main(argc,argv)
int		argc;
char	*argv[];
{
	short	*image1, *outimg;
	long	i, j, jj;
	char	readfile[30], outfile[20];
	short	clg;
	unsigned long	i_end, k;
	double	der, der0;

	HISHeader		his;

	if(argc < 2){   // when bad number of parameters
		printf("Usage is 'his_time hisfile'.\n");
		return(-1);
	}

	sprintf(readfile, ("%s"), argv[1]);

	if ((j = read_hishead(readfile, &his)) != 0){
		printf("error, read his-head - %d", j);
		free(his.comment); 
		return(-1);
	}
	
//	printf("%d\t%d\n", his.n_image1, his.n_image2);
	
	i_end=his.n_image1+65536*his.n_image2;
	if(his.type==6) printf("12 bits %ld images\n", i_end); /* ԈႦĂȂH*/
	if(his.type==3) printf("32 bits %ld images\n", i_end);
	if(his.type==2) printf("16 bits %ld images\n", i_end);
	if(his.type==0) printf("8 bits %ld images\n", i_end);

	printf("reading %s.\n",readfile);

// initialize
	
	if(his.type==2){
		NP=his.width*his.height;
		data=(short *)malloc(NP*sizeof(short));
	}
	if(his.type==6){
		NP=(his.width*his.height)*3/2;
		cdata=(unsigned char *)malloc(NP*sizeof(unsigned char));
	}

	der0=0.0;
	
//open input files
	if((fi = fopen(argv[1],"rb")) == NULL){
		printf("can not open %s for input\n", argv[1]);
		return(-10);
	}
	
	i_end=1000000;
	for(k=0;k<i_end;++k){
// read input images
		if ((j = read_his(&his)) != 0){
			printf("something wrong -- return value is %d", i);
			free(his.comment);
			return(-1);
		}
		der=get_times(&his);

		printf("%d\t",k+1);
//		printf("%g\t%lf\t%d\t%d\t%d\t%d\t%d\t%d\t%d\t%d\n", der, der-der0, his.time_s1,his.time_s2,his.time_s3,his.time_s4,his.time_s5,his.time_s6,his.time_s7,his.time_s8);
		printf("%g\t%lf\n", der, der-der0);
		der0=der;
	}
	
	fprintf(stderr, "\ndone.\n");
//finish!
	free(data); free(cdata);
	free(his.comment);
	fclose(fi);
	return(0);
}
